@@ -1,3 +1,25 @@
+/*
+    tcpcgi.c -- Client portion of rutil_tcpcgi.
+    Copyright (C) 2003  Roy Keene
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+	email: tcpcgi@rkeene.org
+
+*/
+
 /*
  * tcpcgi.c -- Local listener part of reverse-utils::TCP-over-HTTP/CGI:
  *    This needs to do quite a bit of work in the background.
@@ -43,10 +65,11 @@ struct sockinfo {
 };
 
 int alarm_freq=0;
+int use_post=1;
+int local_only=0;
 int poll_time_min=TCPCGI_CLIENT_POLLTIME_MIN;
 int poll_time_max=TCPCGI_CLIENT_POLLTIME_MAX;
 struct sockinfo socks_info[128]; /* Needs to be num of elements as socks_poll in main() */
-#define CLOSE_POLL(arry,idx) { tcpcgi_closesession(arry[idx].fd, http_host, http_port, http_path, socks_info[idx].session); close(arry[idx].fd); arry[idx].fd=-1; socks_info[idx].fd=-1; pollcnt--;  pollpos=idx; }
 
 
 char *hexcode(void *data, int datalen) {
@@ -60,16 +83,22 @@ char *hexcode(void *data, int datalen) {
 	ret=malloc(datalen*3+1);
 
 	for (i=0; i<datalen; i++) {
-#if 0
-		/* if it's a nonprintable charectar, or [%?@&=: ], convert it */
-		/* NOTE!  This may work, but I had issues with it. */
+		if (data_num[i]==' ') {
+			ret[destpos++]='+';
+			continue;
+		}
+#if 1
+		/* if it's a nonprintable charectar, or [%?@&=+: ], convert it */
 		if (!isgraph(data_num[i]) || data_num[i]==' ' || \
 		data_num[i]=='&' || data_num[i]=='=' || \
 		data_num[i]==':' || data_num[i]=='@' || \
-		data_num[i]=='?' || data_num[i]=='%') {
+		data_num[i]=='?' || data_num[i]=='%' || \
+		data_num[i]=='+') {
+#else
+		/* If the charectar is not alphanumeric or and not [+./-], convert it to hex. */
+		/* This is no longer used in favor of a more leanient algorithm. */
+		if (!isalnum(data_num[i]) && data_num[i]!='/' && data_num[i]!='.' && data_num[i]!='~' && data_num[i]!='+') {
 #endif
-		/* If the charectar is not alphanumeric or and not [./-], convert it to hex. */
-		if (!isalnum(data_num[i]) && data_num[i]!='/' && data_num[i]!='.' && data_num[i]!='~') {
 			ret[destpos++]='%';
 			ret[destpos++]=hexabet[(data_num[i]>>4)&0xf];
 			ret[destpos++]=hexabet[data_num[i]&0xf];
@@ -108,14 +137,15 @@ void *tcpcgi_getdata(char *http_host, int http_port, char *http_path, char *cmd,
 
 	fd=createconnection_tcp(http_host, http_port);
 	fp=fdopen(fd, "r+");
-#if 0
-	/* TODO:  Change this to POST */
-	fprintf(fp, "GET http://%s:%i%s?cmd=%s&session=%s&dest=%s&data=%s&ts=%i HTTP/1.0\n\n", http_host, http_port, hex_path, hex_cmd, hex_session, hex_dest, hex_data, (unsigned int) time(NULL));
-#endif
 	snprintf(buf, sizeof(buf), "cmd=%s&session=%s&dest=%s&data=%s&ts=%i", hex_cmd, hex_session, hex_dest, hex_data, (unsigned int) time(NULL));
 	buf[sizeof(buf)-1]='\0';
-	fprintf(fp, "POST %s HTTP/1.0\r\nHost: %s\r\nContent-length: %i\r\n\r\n%s", hex_path, http_host, strlen(buf), buf);
-	PRINTERR("POST http://%s:%i%s?[%i]%s HTTP/1.0\n", http_host, http_port, hex_path, strlen(buf), buf);
+	if (use_post) {
+		fprintf(fp, "POST %s HTTP/1.0\r\nHost: %s\r\nContent-length: %i\r\n\r\n%s", hex_path, http_host, strlen(buf), buf);
+		PRINTERR("POST http://%s:%i%s?[%i]%s HTTP/1.0\n", http_host, http_port, hex_path, strlen(buf), buf);
+	} else {
+		fprintf(fp, "GET http://%s:%i%s?%s HTTP/1.0\r\n\r\n", http_host, http_port, hex_path, buf);
+		PRINTERR("GET http://%s:%i%s?[%i]%s HTTP/1.0\n", http_host, http_port, hex_path, strlen(buf), buf);
+	}
 	fflush(fp);
 
 	if (status) *status=0;
@@ -279,29 +309,34 @@ void sighandler(int sig) {
 
 int print_help(char *msg) {
 	fprintf(stderr, "Usage: tcpcgi -H host -f file -d dest [-P port] [-p port] [-m time]\n");
-	fprintf(stderr, "              [-x time] [-t time] [-b amt] [-rvih]\n");
+	fprintf(stderr, "              [-x time] [-t time] [-b amt] [-riglvh]\n");
 	fprintf(stderr, "    Options:\n");
 	fprintf(stderr, "         -H host      HTTP Server which has tcpcgi.cgi.\n");
 	fprintf(stderr, "         -f file      Full URL-relative on HTTP server to tcpcgi.cgi,\n");
 	fprintf(stderr, "                      including tcpcgi.cgi.\n");
 	fprintf(stderr, "         -d dest      Destination to for HTTP server to connect to and\n");
-	fprintf(stderr, "                      ultimately proxy to (must be in the form of host:port).\n");
-	fprintf(stderr, "         -P port      Port to connect to HTTP server on (80 is the default).\n");
-	fprintf(stderr, "         -p port      Port to listen for connections on locally (%i is the\n", TCPCGI_CLIENT_PORT);
-	fprintf(stderr, "                      default).\n");
+	fprintf(stderr, "                      proxy to (must be in the form of host:port).\n");
+	fprintf(stderr, "         -P port      Port to connect to HTTP server on.\n");
+	fprintf(stderr, "         -p port      Port to listen for connections on locally (%i is\n", TCPCGI_CLIENT_PORT);
+	fprintf(stderr, "                      the default).\n");
 	fprintf(stderr, "         -m time      Lowest amount of time (in milliseconds) to check for\n");
 	fprintf(stderr, "                      data from HTTP server (%i is the default).\n", TCPCGI_CLIENT_POLLTIME_MIN);
-	fprintf(stderr, "         -x time      Greatest amount of time (in milliseconds) to check for\n");
-	fprintf(stderr, "                      data from HTTP server (%i is the default).\n", TCPCGI_CLIENT_POLLTIME_MAX);
+	fprintf(stderr, "         -x time      Greatest amount of time (in milliseconds) to check\n");
+	fprintf(stderr, "                      for data from HTTP server (%i is the default).\n", TCPCGI_CLIENT_POLLTIME_MAX);
 	fprintf(stderr, "         -t time      Specify that the amount of time for polling be fixed\n");
 	fprintf(stderr, "                      at this value (in ms).\n");
-	fprintf(stderr, "         -b amt       Backoff amount, if less than 5, specifies backoff should\n");
-	fprintf(stderr, "                      multiply the backoff time by this number, otherwise it's\n");
-	fprintf(stderr, "                      incremented by this amount (2 is the default).\n");
+	fprintf(stderr, "         -b amt       Backoff amount, if less than 5, specifies backoff\n");
+	fprintf(stderr, "                      should multiply the backoff time by this number,\n");
+	fprintf(stderr, "                      otherwise it's incremented by this amount (2 is the\n"); 
+	fprintf(stderr, "                      default).\n");
 	fprintf(stderr, "         -r           Specify that the backoff algorithm reset when remote\n");
 	fprintf(stderr, "                      activity occurs, instead of decrementing.\n");
 	fprintf(stderr, "         -i           Specify that the backoff algorithm should respond to\n");
 	fprintf(stderr, "                      local traffic as well as remote.\n");
+	fprintf(stderr, "         -g           Toggle between HTTP get and HTTP post (%s is\n", use_post?"POST":"GET");
+	fprintf(stderr, "                      the default).\n");
+	fprintf(stderr, "         -l           Bind to %s only for listening (%s is the\n", local_only?"ALL":"LOCALHOST", local_only?"LOCAL":"ALL");
+	fprintf(stderr, "                      default).\n");
 	fprintf(stderr, "         -v           Print version (%s) and exit.\n", TCPCGI_VERSION);
 	fprintf(stderr, "         -h           This help information.\n");
 	if (msg) {
@@ -310,8 +345,19 @@ int print_help(char *msg) {
 	return(1);
 }
 
+#ifdef CLOSE_POLL
+#undef CLOSE_POLL
+#endif
+#define CLOSE_POLL(idx) { \
+	tcpcgi_closesession(socks_poll[idx].fd, http_host, http_port, http_path, socks_info[idx].session); \
+	close(socks_poll[idx].fd); \
+	socks_poll[idx].fd=-1; \
+	socks_info[idx].fd=-1; \
+	pollcnt--; \
+	pollpos=idx; \
+}
 int main(int argc, char **argv) {
-	struct pollfd socks_poll[128];
+	struct pollfd socks_poll[sizeof(socks_info)/sizeof(struct sockinfo)];
 	struct sockaddr addr;
 	signed int pollpos=0;
 	signed int status=0;
@@ -334,7 +380,7 @@ int main(int argc, char **argv) {
 	pid_t chpid;
 #endif
 
-	while ((ch=getopt(argc, argv, "H:f:P:p:d:m:x:t:b:vrih"))!=-1) {
+	while ((ch=getopt(argc, argv, "H:f:P:p:d:m:x:t:b:vgrilh"))!=-1) {
 		switch (ch) {
 			case 'H':
 				http_host=strdup(optarg);
@@ -379,11 +425,16 @@ int main(int argc, char **argv) {
 			case 'i':
 				poll_backoff_ireset=!poll_backoff_ireset;
 				break;
+			case 'g':
+				use_post=!use_post;
+				break;
 			case 'v':
 				printf("Reverse utilities::TCP-over-HTTP/CGI v%s\n", TCPCGI_VERSION);
 				return(0);
 				break;
-				
+			case 'l':
+				local_only=!local_only;
+				break;
 			case ':':
 			case '?':
 			case 'h':
@@ -409,7 +460,7 @@ int main(int argc, char **argv) {
 
 	signal(SIGALRM, sighandler);
 	signal(SIGPIPE, SIG_IGN);
-	masterfd=createlisten(listen_port);
+	masterfd=createlisten(listen_port, local_only);
 	if (masterfd<0) {
 		PERROR("createlisten()");
 		return(1);
@@ -481,7 +532,7 @@ int main(int argc, char **argv) {
 		for (i=0; i<pollmax; i++) {
 			if (socks_poll[i].revents&(POLLERR|POLLHUP|POLLNVAL)) {
 				/* Socket is broke, close it. */
-				CLOSE_POLL(socks_poll, i);
+				CLOSE_POLL(i);
 				PRINTERR("Closing socket [idx=%i, fd=%i], due to errors.", i, fd);
 				num_events--;
 				if (num_events==0) break;
@@ -528,7 +579,7 @@ int main(int argc, char **argv) {
 				tcpcgi_getdata(http_host, http_port, http_path, "open", socks_info[pollpos].session, hostport, NULL, NULL, &status);
 				if (status<0) {
 					PRINTERR("Error creating session on remote end.");
-					CLOSE_POLL(socks_poll, pollpos);
+					CLOSE_POLL(pollpos);
 					continue;
 				}
 
@@ -545,12 +596,12 @@ int main(int argc, char **argv) {
 				if (x<0) PERROR("read");
 				if (x==0) {
 					PRINTERR("EOF from [idx=%i, fd=%i].", i, fd);
-					CLOSE_POLL(socks_poll, i);
+					CLOSE_POLL(i);
 					continue;
 				}
 				if ((rv=tcpcgi_handledata(http_host, http_port, http_path, socks_info[i].session, fd, buf, x))<0) {
 					PRINTERR("Unable to put data.");
-					CLOSE_POLL(socks_poll, i);
+					CLOSE_POLL(i);
 					continue;
 				}
 				if (poll_backoff_ireset) {
