@@ -1,3 +1,25 @@
+/*
+    tcpcgi.cgi.c -- HTTP/CGI portion of rutil_tcpcgi.
+    Copyright (C) 2003  Roy Keene
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+	email: tcpcgi@rkeene.org
+
+*/
+
 /*
  * tcpcgi.cgi.c -- CGI part of reverse-utils::TCP-over-HTTP/CGI:
  *    This simply needs to relay everything receieved to the daemon
@@ -17,8 +39,10 @@
 #include <fcntl.h>
 #include <ctype.h>
 
-#include "tcpcgid.h"
 #include "tcpcgi.h"
+#ifndef TCPCGID_STANDALONE
+#include "tcpcgid.h"
+#endif
 #include "tcpnet.h"
 #define CACHE_DEBUG
 #include "cache.h"
