@@ -1,3 +1,25 @@
+/*
+    tcpcgid.c -- Server/daemon portion of rutil_tcpcgi.
+    Copyright (C) 2003  Roy Keene
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+	email: tcpcgi@rkeene.org
+
+*/
+
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -16,7 +38,15 @@
 #define CACHE_DEBUG
 #include "cache.h"
 
-#define CLOSE_POLL(arry,idx) { close(arry[idx].fd); tcpcgi_handledata(arry[idx].fd, NULL, 0); arry[idx].fd=-1; pollcnt--; pollpos=idx; }
+#define CLOSE_POLL(idx) { \
+	close(socks_poll[idx].fd); \
+	tcpcgi_handledata(socks_poll[idx].fd, NULL, 0); \
+	socks_poll[idx].fd=-1; \
+	pollcnt--; \
+	pollpos=idx; \
+}
+
+
 static cache_t *sess=NULL;
 
 struct conninfo {
@@ -322,7 +352,7 @@ int MAIN_FUNC(void) {
 	/* Note, this code is duplicated below with the OPPOSITE condition */
 	signal(SIGPIPE, SIG_IGN);
 
-	masterfd=createlisten(TCPCGI_DAEMON_PORT);
+	masterfd=createlisten(TCPCGI_DAEMON_PORT, 1);
 	if (masterfd<0) {
 		PERROR("createlisten()");
 		return(-1);
@@ -351,7 +381,7 @@ int MAIN_FUNC(void) {
 	/* If we're not in stand-alone mode, do this now (since we didn't do it above). */
 	signal(SIGPIPE, SIG_IGN);
 
-	masterfd=createlisten(TCPCGI_DAEMON_PORT);
+	masterfd=createlisten(TCPCGI_DAEMON_PORT, 1);
 	if (masterfd<0) {
 		PERROR("createlisten()");
 		RET_FROM_MAIN(-1);
@@ -383,7 +413,7 @@ int MAIN_FUNC(void) {
 		for (i=0; i<pollmax; i++) {
 			if (socks_poll[i].revents&(POLLERR|POLLHUP|POLLNVAL)) {
 				/* Socket is broke, close it. */
-				CLOSE_POLL(socks_poll, i);
+				CLOSE_POLL(i);
 				PRINTERR("Closing socket [idx=%i, fd=%i], due to errors.", i, fd);
 				num_events--;
 				if (num_events==0) break;
@@ -430,7 +460,7 @@ int MAIN_FUNC(void) {
 				if (x<0) PERROR("read");
 				if (x==0) {
 					PRINTERR("EOF from [idx=%i, fd=%i].", i, fd);
-					CLOSE_POLL(socks_poll, i);
+					CLOSE_POLL(i);
 					continue;
 				}
 				if (tcpcgi_handledata(fd, buf, x)<0) {
