@@ -21,7 +21,7 @@
 #ifndef _DACT_NET_H 
 #define _DACT_NET_H
 
-int createlisten(int port);
+int createlisten(int port, int localonly);
 void closeconnection(int sockfd);
 int createconnection_tcp(char *host, int port);
 #endif
